from zope.interface import implements
from zope.traversing.interfaces import ITraversable
from zope.component import getUtility
from zope.component import adapts

from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.AdvancedQuery import Indexed, Le, Eq, In
from Acquisition import Implicit
from DateTime import DateTime

from plone.i18n.normalizer.interfaces import IURLNormalizer
from plone.registry.interfaces import IRegistry

from collective.chimpfeed.interfaces import IFeedSettings
from collective.chimpfeed.interfaces import IBrowserLayer


def make_urls(feeds):
    filtered = filter(None, feeds or ())
    normalize = getUtility(IURLNormalizer).normalize

    return dict((
        ("%s.rss" % normalize(feed), feed)
        for feed in filtered
        ))


class Feed(Implicit):
    template = ViewPageTemplateFile("feed.pt")

    max_items = 25

    def __init__(self, context, request, name):
        self.context = context
        self.request = request
        self.name = name

    def HEAD(self, *args):
        """Handle HEAD-request."""

        response = self.template()
        self.request.response.setHeader('Content-Length', len(response))

        # Actually return an empty string
        return u""

    def get_brains(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        today = DateTime()

        query = In('review_state', ('visible', 'published')) & \
                Eq('chimpfeeds', self.name) & \
                Eq('feedModerate', True) & \
                Le('feedSchedule', today)

        brains = catalog.evalAdvancedQuery(query)
        return tuple(brains)

    def index_html(self):
        """Publish RSS-feed."""

        return self.template()


class Feeds(Implicit):
    def __getitem__(self, name):
        settings = getUtility(IRegistry).forInterface(IFeedSettings)
        urls = make_urls(settings.feeds)
        name = urls[name]
        return Feed(self, self.REQUEST, name)


class FeedTraverser(object):
    implements(ITraversable)
    adapts(ISiteRoot, IBrowserLayer)

    def __init__(self, context, request=None):
        self.context = context
        self.request = request

    def traverse(self, name, ignore):
        feeds = Feeds()

        if name:
            return feeds[name]

        return feeds



import greatape

from zope.interface import Interface
from zope import schema

from plone.portlets.interfaces import IPortletDataProvider

from collective.chimpfeed import MessageFactory as _
from collective.chimpfeed import logger


def check_api_key(api_key):
    if not api_key:
        return True

    api = greatape.MailChimp(api_key)

    try:
        api(method="ping")
    except greatape.MailChimpError:
        pass
    except ValueError, exc:
        logger.warn(exc)
    else:
        return True

    return False


class INameSplitter(Interface):
    def split_name(fullname):
        """Return (fname, lname) tuple."""


class IBrowserLayer(Interface):
    """Theme-specific browser-layer."""


class IFeedSettings(Interface):
    mailchimp_api_key = schema.TextLine(
        title=_(u"MailChimp API key"),
        description=_(u"This key is associated to your "
                      u"account with MailChimp."),
        required=False,
        default=u"",
        constraint=check_api_key,
        )

    feeds = schema.List(
        title=_(u"Feeds"),
        description=_(u"Use this field to set up your feeds. "
                      u"Give each feed a title such as \"News\"; "
                      u"one feed per line."),
        required=False,
        value_type=schema.TextLine(),
        )


class ISubscriptionFormSettings(Interface):
    mailinglist = schema.Choice(
        title=_(u"Mailinglist"),
        description=_(u"Select a mailinglist for this portlet."),
        vocabulary="collective.chimpfeed.vocabularies.Lists",
        required=True,
        )

    interest_groupings = schema.Tuple(
        title=_(u"Interest groups"),
        description=_(u"Select interest groups. "
                      u"Note that all interests within the selected "
                      u"groups will appear in the portlet."),
        value_type=schema.Choice(
            vocabulary="collective.chimpfeed.vocabularies.InterestGroupings",
            ),
        required=False,
        )

    interest_groups = schema.Tuple(
        title=_(u"Interests"),
        description=_(u"Select interests. "
                      u"The number listed in each parenthesis is "
                      u"the current number of subscribers with "
                      u"that interest."),
        value_type=schema.Choice(
            vocabulary="collective.chimpfeed.vocabularies.InterestGroups",
            ),
        required=False,
        )


class IModerationPortlet(IPortletDataProvider):
    pass


class ISubscriptionPortlet(IPortletDataProvider, ISubscriptionFormSettings):
    heading = schema.TextLine(
        title=_(u"Title"),
        description=_(u"Provide a title for the portlet."),
        required=True,
        default=u"Sign up",
        )


class IControlPanel(IFeedSettings):
    urls = schema.Tuple(
        title=_(u"URLs"),
        description=_(u"These correspond to the feeds given above. "
                      u"Copy the URL into MailChimp when setting up one "
                      u"or more RSS-campaigns."),
        required=False,
        value_type=schema.ASCIILine(),
        )



from zope.formlib import form
from zope.interface import implements
from zope.interface import alsoProvides
from zope.interface import noLongerProvides
from plone.z3cform.interfaces import IWrappedForm
from plone.app.z3cform.interfaces import IPloneFormLayer
from z3c.form.interfaces import IFormLayer
from plone.app.portlets.portlets import base
from five.formlib.formbase import FormBase
from zope.security import checkPermission

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from collective.chimpfeed.form import SubscribeForm
from collective.chimpfeed.form import ModerationForm
from collective.chimpfeed.interfaces import ISubscriptionPortlet
from collective.chimpfeed.interfaces import IModerationPortlet
from collective.chimpfeed import MessageFactory as _


class ModerationPortletAssignment(base.Assignment):
    implements(IModerationPortlet)

    heading = _(u"Moderation")

    title = _(u"Feed moderation")


class SubscriptionPortletAssignment(base.Assignment):
    implements(ISubscriptionPortlet)

    def __init__(self, **kwargs):
        for name, value in kwargs.items():
            self.__dict__[name] = value

    @property
    def title(self):
        return _("Subscribe: ${title}", mapping={'title': self.heading})


class FormPortletRenderer(base.Renderer):
    render = ViewPageTemplateFile('portlet.pt')

    name = ""

    def render_form(self):
        provided = IPloneFormLayer.providedBy(self.request)
        noLongerProvides(self.request, IPloneFormLayer)
        alsoProvides(self.request, IFormLayer)
        try:
            form = self.create_form()
            alsoProvides(form, IWrappedForm)
            form.update()
            return form.render()
        finally:
            noLongerProvides(self.request, IFormLayer)
            if provided:
                alsoProvides(self.request, IPloneFormLayer)


class ModerationPortletRenderer(FormPortletRenderer):
    name = "Moderation"

    @property
    def available(self):
        return checkPermission("chimpfeed.Moderate", self.context)

    def create_form(self):
        context = self.data.__of__(self.context)
        return ModerationForm(context, self.request)


class SubscriptionPortletRenderer(FormPortletRenderer):
    def create_form(self):
        context = self.data.__of__(self.context)
        return SubscribeForm(context, self.request)


class ModerationPortletAddForm(base.NullAddForm):
    def create(self):
        return ModerationPortletAssignment()


class SubscriptionPortletForm(FormBase):
    name = "Subscribe"

    form_fields = form.Fields(ISubscriptionPortlet)
    description = _(u"This portlet shows a subscription form.")

    def setUpWidgets(self, **kwargs):
        super(SubscriptionPortletForm, self).setUpWidgets(**kwargs)

        widget = self.widgets['mailinglist']
        if len(widget.vocabulary):

            mailinglist = ISubscriptionPortlet.providedBy(self.context) and \
                          self.context.mailinglist or widget.hasInput() \
                          and widget.getInputValue()

            if mailinglist:
                pass
            else:
                # Make a default choice
                token = tuple(widget.vocabulary)[-1].token
                widget.setRenderedValue(token)
        else:
            self.widgets['interest_groups'].hidden = True
            self.widgets['interest_groupings'].hidden = True


class SubscriptionPortletAddForm(SubscriptionPortletForm, base.AddForm):
    label = _(u"Add Subscription Portlet")

    def create(self, data):
        return SubscriptionPortletAssignment(**data)


class SubscriptionPortletEditForm(SubscriptionPortletForm, base.EditForm):
    label = _(u"Edit Subscription Portlet")



import time
import math
import greatape
import cgi

from Products.statusmessages.interfaces import IStatusMessage
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName

from ZODB.utils import u64
from DateTime import DateTime

from plone.registry.interfaces import IRegistry
from plone.memoize.ram import cache
from plone.memoize.instance import memoizedproperty
from plone.memoize.volatile import DontCache

from zope.i18n import negotiate
from zope.i18n import translate
from zope.component import getUtility
from zope.component import queryUtility
from zope.interface import Interface
from zope.interface import implements
from zope.schema.vocabulary import SimpleVocabulary
from zope import schema

from z3c.form import button
from z3c.form import field
from z3c.form import form
from z3c.form.widget import SequenceWidget
from z3c.form.widget import FieldWidget
from z3c.form.interfaces import IErrorViewSnippet
from z3c.form.browser.checkbox import CheckBoxFieldWidget

from collective.chimpfeed import MessageFactory as _
from collective.chimpfeed import logger
from collective.chimpfeed.interfaces import IFeedSettings
from collective.chimpfeed.interfaces import INameSplitter
from collective.chimpfeed.vocabularies import interest_groups_factory
from collective.chimpfeed.splitters import GenericNameSplitter


def create_groupings(groups):
    groupings = {}
    for grouping_id, name in groups:
        groupings.setdefault(grouping_id, []).append(name)
    return groupings


def cache_on_get_for_an_hour(method, self):
    if self.request.method != 'GET':
        raise DontCache

    return time.time() // (60 * 60), self.id


class IModeration(Interface):
    items = schema.Tuple(
        value_type=schema.Choice(
            vocabulary="collective.chimpfeed.vocabularies.ScheduledItems",
            ),
        required=False,
        )


class ISubscription(Interface):
    name = schema.TextLine(
        title=_(u"Name"),
        required=True,
        )

    email = schema.TextLine(
        title=_(u"E-mail"),
        required=True,
        )

    interests = schema.Tuple(
        value_type=schema.Choice(
            vocabulary="collective.chimpfeed.vocabularies.InterestGroups",
            ),
        required=False,
        )


class ModerationWidget(SequenceWidget):
    template = ViewPageTemplateFile("moderate.pt")

    @classmethod
    def factory(cls, field, request):
        return FieldWidget(field, cls(request))

    def render(self):
        return self.template()

    def _localize_time(self, time, long_format):
        util = getToolByName(self.context, 'translation_service')

        return util.ulocalized_time(
            time, long_format=long_format,
            context=self.context, request=self.request,
            domain='plonelocales'
            )

    @memoizedproperty
    def entries(self):
        entries = []
        catalog = self.context.portal_catalog

        for term in self.terms:
            rid = term.value
            entry = catalog.getMetadataForRID(rid)
            entries.append(entry)

        return entries

    @memoizedproperty
    def action_required(self):
        for entry in self.entries:
            if not entry['feedModerate']:
                return True

    @property
    def groups(self):
        last = None
        groups = []
        entries = []
        today = DateTime()

        for entry in self.entries:
            date = entry['feedSchedule']

            days = int(math.floor(date - today))
            if days != last:
                entries = []

                # To-Do: Use Plone's date formatters
                if days == -1:
                    name = _(u"Today")
                elif days < 7:
                    abbr = date.strftime("%a")
                    name = translate(
                        'weekday_%s' % abbr.lower(),
                        domain="plonelocales",
                        context=self.request
                        )
                else:
                    name = self._localize_time(date, False)

                groups.append({
                    'date': name,
                    'entries': entries,
                    })

            entries.append(entry)

        return groups


class InterestsWidget(SequenceWidget):
    template = ViewPageTemplateFile("interests.pt")

    @cache(cache_on_get_for_an_hour)
    def render(self):
        return self.template()

    @classmethod
    def factory(cls, field, request):
        return FieldWidget(field, cls(request))

    @memoizedproperty
    def groupings(self):
        return tuple(interest_groups_factory.get_groupings())

    def renderInterestGroups(self):
        groups = create_groupings(self.context.interest_groups)

        terms = []
        for grouping in self.groupings:
            grouping_id = grouping['id']
            names = groups.get(grouping_id, ())
            for group in grouping['groups']:
                name = group['name']
                if name in names:
                    term = interest_groups_factory.get_term_for_group(
                        name, grouping_id
                        )
                    terms.append(term)

        return self.renderChoices(terms)

    def renderInterestGroupings(self):
        filtered = (
            grouping for grouping in self.groupings
            if grouping['id'] in self.context.interest_groupings
            )

        rendered = []

        for grouping in filtered:
            terms = tuple(interest_groups_factory.get_terms_for_grouping(
                grouping, count=False
                ))

            # Create label from the grouping name; we simply lowercase
            # it.
            label = _(u"Choose ${name}:", mapping={
                'name': cgi.escape(grouping['name'].lower())
                })

            result = self.renderChoices(terms, label)
            rendered.append(result)

        return u"\n".join(rendered)

    def renderChoices(self, terms, label=None):
        choice = schema.Choice(
            vocabulary=SimpleVocabulary(terms),
            )

        field = self.field.bind(self.context)
        field.value_type = choice

        widget = CheckBoxFieldWidget(field, self.request)
        widget.name = self.name
        widget.id = self.id

        widget.update()
        result = widget.render()

        if label is not None:
            result = u"<fieldset><legend>%s</legend>%s</fieldset>" % (
                translate(label, context=self.request), result
                )

        return result


class SchemaErrorSnippet(object):
    implements(IErrorViewSnippet)

    def __init__(self, error, request, widget, field, form, content):
        self.error = error

    def update(self):
        pass

    def render(self):
        return _(self.error.__doc__)


class BaseForm(form.Form):
    ignoreContext = True

    @property
    def action(self):
        # Use the parent object's URL instead of the current request
        # URL; this avoids adding a view name to the end of the path.
        return self.context.aq_parent.absolute_url()

    @property
    def prefix(self):
        # Make sure form prefix is unique
        return 'form-%d-%d.' % (
            u64(self.context._p_oid),
            int(self.context._p_mtime) % 10000
            )


class ModerationForm(BaseForm):
    fields = field.Fields(IModeration)

    fields['items'].widgetFactory = ModerationWidget.factory

    ignoreContext = True

    @button.buttonAndHandler(_(u'Approve'))
    def handleApprove(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        catalog = self.context.portal_catalog
        for rid in data['items'] or ():
            metadata = catalog.getMetadataForRID(rid)
            for brain in self.context.uid_catalog(UID=metadata['UID']):
                obj = brain.getObject()
                obj.getField('feedModerate').set(obj, True)
                obj.reindexObject(['feedModerate'])

    def update(self):
        super(ModerationForm, self).update()

        # No reason to show a button if no action is required (or
        # possible).
        if not self.widgets['items'].action_required:
            del self.actions['approve']


class SubscribeForm(BaseForm):
    fields = field.Fields(ISubscription)

    fields['interests'].widgetFactory = InterestsWidget.factory

    @button.buttonAndHandler(_(u'Register'))
    def handleApply(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        settings = getUtility(IRegistry).forInterface(IFeedSettings)
        api_key = settings.mailchimp_api_key

        try:
            list_id = self.context.mailinglist

            email = data['email']
            name = data['name']
            interests = data['interests']

            if api_key:
                api = greatape.MailChimp(api_key, debug=False)

                # Negotiate language
                language = negotiate(self.request)

                # Split full name into first (given) and last name.
                fname, lname = queryUtility(
                    INameSplitter, name=language, default=GenericNameSplitter
                    ).split_name(name)

                try:
                    result = api(
                        method="listSubscribe", id=list_id,
                        email_address=email,
                        update_existing=True,
                        replace_interests=False,
                        merge_vars={
                            'FNAME': fname.encode('utf-8'),
                            'LNAME': lname.encode('utf-8'),
                            'GROUPINGS': [
                                dict(
                                    id=grouping_id,
                                    groups=",".join(
                                        group.\
                                        encode('utf-8').\
                                        replace(',', '\\,')
                                        for group in group_names
                                        ),
                                    )
                                for (grouping_id, group_names) in
                                create_groupings(interests).items()
                                ]
                            },
                        )
                except greatape.MailChimpError, exc:
                    logger.warn(exc.msg)
                else:
                    if result:
                        return IStatusMessage(self.request).addStatusMessage(
                            _(u"Thank you for signing up. We'll send you a "
                              u"confirmation message by e-mail shortly."),
                            "info"
                            )

            IStatusMessage(self.request).addStatusMessage(
                _(u"An error occurred while processing your "
                  u"request to sign up for ${email}. "
                  u"Please try again!", mapping={'email': email}),
                "error")

        finally:
            self.request.response.redirect(self.action)



import logging

from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from Acquisition import ImplicitAcquisitionWrapper

from zope.interface import implements
from zope.component import adapts

from plone.z3cform import layout
from plone.app.registry.browser import controlpanel

from collective.chimpfeed.interfaces import IFeedSettings
from collective.chimpfeed.interfaces import IControlPanel
from collective.chimpfeed import MessageFactory as _
from collective.chimpfeed.feeds import make_urls

from z3c.form import field
from z3c.form import widget
from z3c.form.interfaces import IDataConverter

logger = logging.getLogger("collective.chimpfeed.controlpanel")


class ControlPanelAdapter(object):
    adapts(IFeedSettings)
    implements(IControlPanel)

    def __init__(self, settings):
        self.settings = settings

    @property
    def urls(self):
        portal_url = getToolByName(self.settings, 'portal_url')()
        urls = make_urls(self.settings.feeds)
        return [
            "%s/++chimpfeeds++/%s" % (portal_url, url)
            for url in urls
            ]


class UrlsWidget(widget.Widget):
    implements(IDataConverter)

    render = ViewPageTemplateFile("urls.pt")

    @classmethod
    def factory(cls, field, request):
        return widget.FieldWidget(field, cls(request))

    def toWidgetValue(self, value):
        return value


class ControlPanelEditForm(controlpanel.RegistryEditForm):
    schema = IFeedSettings
    fields = field.Fields(IControlPanel)
    label = _(u"MailChimp settings")
    description = _(
        u"Please configure an API-key and define one or more "
        u"feeds."
        )

    fields['urls'].mode = "display"
    fields['urls'].widgetFactory = UrlsWidget.factory

    def updateActions(self):
        # This prevents a redirect to the main control panel page
        self.request.response.setStatus(200, lock=True)

        super(ControlPanelEditForm, self).updateActions()

    def render(self):
        if not self.getContent().feeds:
            del self.widgets['urls']
        else:
            self.widgets['urls'].update()

        return super(ControlPanelEditForm, self).render()

    def getContent(self):
        content = super(ControlPanelEditForm, self).getContent()
        return ImplicitAcquisitionWrapper(content, self.context)


ControlPanel = layout.wrap_form(
    ControlPanelEditForm,
    controlpanel.ControlPanelFormWrapper
    )



import greatape
import base64
import time

from zope.interface import implements
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from plone.registry.interfaces import IRegistry
from plone.i18n.normalizer.interfaces import IIDNormalizer
from plone.memoize.ram import cache

from collective.chimpfeed.interfaces import IFeedSettings
from collective.chimpfeed import logger

from Products.AdvancedQuery import Indexed, Ge
from DateTime import DateTime


def stub_api(**kwargs):
    return ()


class VocabularyBase(object):
    implements(IVocabularyFactory)


class ScheduledItems(VocabularyBase):
    def __call__(self, context):
        terms = []
        today = DateTime()
        today = DateTime(today.year(), today.month(), today.day())

        query = Indexed('chimpfeeds') & Ge('feedSchedule', today)
        brains = context.portal_catalog.evalAdvancedQuery(
            query, (('feedSchedule', 'desc'), ))

        for brain in brains:
            rid = brain.getRID()
            terms.append(
                SimpleTerm(rid, brain.UID, brain.Title)
                )

        return SimpleVocabulary(terms)


class FeedVocabulary(VocabularyBase):
    def __call__(self, context):
        normalize = getUtility(IIDNormalizer).normalize
        settings = getUtility(IRegistry).forInterface(IFeedSettings)

        return SimpleVocabulary([
            SimpleTerm(feed, normalize(feed), feed)
            for feed in settings.feeds or ()
            ])


class MailChimpVocabulary(VocabularyBase):
    def __call__(self, context):
        generator = self.get_terms(context)
        terms = tuple(generator)
        return SimpleVocabulary(terms)

    @property
    def api(self):
        settings = getUtility(IRegistry).forInterface(IFeedSettings)
        api_key = settings.mailchimp_api_key

        if api_key:
            api = greatape.MailChimp(api_key)

            def api(api=api, **kwargs):
                try:
                    return api(**kwargs)
                except greatape.MailChimpError, exc:
                    # http://apidocs.mailchimp.com/api/1.3/exceptions.field.php
                    if exc.code <= 0:
                        logger.critical(exc)
                    elif exc.code < 120:
                        logger.warn(exc)
                    elif exc.code < 200:
                        logger.info(exc)
                except TypeError, exc:
                    logger.warn(exc)

                return ()

            return api

        return stub_api

    @cache(lambda *args: time.time() // (5 * 60))
    def get_lists(self):
        results = []
        for result in self.api(method="lists"):
            results.append((result['id'], result['name']))

        return results

    @cache(lambda *args: time.time() // (5 * 60))
    def get_groupings(self):
        results = []

        for list_id, list_name in self.get_lists():
            groupings = self.api(
                method="listInterestGroupings", id=list_id
                )
            for grouping in groupings:
                results.append(grouping)

        return results


class ListVocabulary(MailChimpVocabulary):
    def get_terms(self, context):
        for value, name in self.get_lists():
            yield SimpleTerm(value, value, name)


class InterestGroupVocabulary(MailChimpVocabulary):
    def get_terms(self, context):
        groupings = self.get_groupings()

        for grouping in groupings:
            for term in self.get_terms_for_grouping(grouping):
                yield term

    def get_terms_for_grouping(self, grouping, count=True):
        for group in grouping['groups']:
            term = self.get_term_for_group(group['name'], grouping['id'])

            if count:
                term.title += " (%d)" % group['subscribers']

            yield term

    def get_term_for_group(self, name, grouping_id):
        token = "%s-%s" % (
            grouping_id,
            base64.urlsafe_b64encode(name.encode('utf-8'))
            )

        value = (grouping_id, name)
        return SimpleTerm(value, token, name)


class InterestGroupingVocabulary(MailChimpVocabulary):
    def get_terms(self, context):
        groupings = self.get_groupings()

        for grouping in groupings:
            value = token = grouping['id']
            groups = grouping['groups']
            name = grouping['name']

            if groups:
                name += u" (%s)" % ", ".join(
                    group['name'] for group in groups
                    )

            yield SimpleTerm(value, token, name)


feeds_factory = FeedVocabulary()
lists_factory = ListVocabulary()
interest_groupings_factory = InterestGroupingVocabulary()
interest_groups_factory = InterestGroupVocabulary()
scheduled_items = ScheduledItems()



from zope.interface import implements
from zope.component import adapts
from archetypes.schemaextender.interfaces import ISchemaExtender
from archetypes.schemaextender.field import ExtensionField
from plone.indexer.decorator import indexer

from Products.Archetypes import atapi
from Products.Archetypes.interfaces import IBaseContent

from collective.chimpfeed import MessageFactory as _
from collective.chimpfeed.permissions import MODERATE_PERMISSION


@indexer(IBaseContent)
def feed_indexer(context):
    feeds = context.getField('feeds').get(context)
    return [unicode(feed, 'utf-8') for feed in feeds]


class ScheduleField(ExtensionField, atapi.DateTimeField):
    def set(self, instance, value, **kwargs):
        previous = self.get(instance, **kwargs)

        # Set moderation to `False` to prompt an editor to moderate
        # new date.
        if previous == value:
            return

        instance.getField('feedModerate').set(instance, False)
        super(ScheduleField, self).set(instance, value, **kwargs)


class LinesField(ExtensionField, atapi.LinesField):
    pass


class ModerationField(ExtensionField, atapi.BooleanField):
    pass


class FeedExtender(object):
    implements(ISchemaExtender)
    adapts(IBaseContent)

    fields = (
        LinesField(
            name="feeds",
            required=False,
            multivalued=1,
            schemata="settings",
            widget=atapi.MultiSelectionWidget(
                label=_("Feeds"),
                description=_(u"If you want this content item published "
                              u"to a news feed, select one or more from "
                              u"the list below."),
                ),
            vocabulary_factory="collective.chimpfeed.vocabularies.Feeds",
            ),

        ScheduleField(
            'feedSchedule',
            schemata="settings",
            languageIndependent=True,
            required=False,
            default=None,
            widget=atapi.CalendarWidget(
                show_hm=False,
                starting_year=2012,
                ending_year=2015,
                label=_(u'Schedule feed date'),
                description=_(u'The item will be scheduled to be included '
                              u'in feeds from this date. Note that this '
                              u'is subject to editor moderation.'),
                ),
            ),

        ModerationField(
            'feedModerate',
            schemata="settings",
            default=False,
            enforceVocabulary=1,
            write_permission=MODERATE_PERMISSION,
            widget=atapi.BooleanWidget(
                label=_(u'Approve schedule'),
                description=_(u'Select this option to approve schedule.'),
                ),
            ),
        )

    def __init__(self, context):
        self.context = context

    def getFields(self):
        return self.fields
